/*******************************************************************************
* Functions used to support updating values in a page being displayed in the
* browser. Values to be updated are wrapped in spans with IDs. The updates
* variable is populated with a data structure that includes the new values and
* processUpdates() is called to unload the new values.
*
* Note that this requires that the page contains an allowRefresh variable,
* which determines whether these functions do anything. This is added by
* header.js
*******************************************************************************/
var updates;
var pause = false;
var lastChangeTime = -1;
var loadCompleteTime = -1;
var minimumTime = -1;
var missed = 0;
var isThrottled = false;

function togglePause()
{
  if (pause)
  {
    playUpdates();
  }
  else
  {
    pauseUpdates();
  }
}   

function showPauseUpdates()
{
  if (updates.interval == 0)
  {
    document.getElementById("playUpdatesButton").style.display = "inline";
    document.getElementById("playUpdatesLink").style.display = "inline";    
  }
  else
  {
    document.getElementById("pauseUpdatesButton").style.display = "inline";  
    document.getElementById("pauseUpdatesLink").style.display = "inline";
  }
}

function playUpdates()
{
  focusMenu();
  document.getElementById("playUpdatesButton").style.display = "none";
  document.getElementById("pauseUpdatesButton").style.display = "inline";
  document.getElementById("playUpdatesLink").style.display = "none";
  document.getElementById("pauseUpdatesLink").style.display = "inline";  
  pause = false;
  updates.interval = 5000;
  processUpdates();
  tick();
}

function pauseUpdates()
{
  focusMenu();
  document.getElementById("playUpdatesButton").style.display = "inline";
  document.getElementById("pauseUpdatesButton").style.display = "none";
  document.getElementById("playUpdatesLink").style.display = "inline";
  document.getElementById("pauseUpdatesLink").style.display = "none";  
  pause = true;
  updates.interval = 0;
  setRefreshEnabled();
  processUpdates();
}

function focusMenu()
{
  var element = document.getElementById("menuItem_0");
  if (typeof element != "undefined")
  {
    element.focus();
  }
}

function processUpdates()
{
  if (allowRefresh && !pause)
  {
    if (isThrottled)
    {
      loadCompleteTime = -1;
      lastChangeTime = new Date().getTime();
      missed = 0;
    }
    
    if (typeof updates == "undefined")
    {
      return;
    }
    
    if (typeof updates.changes != "undefined")
    {   
      for (i = 0; i < updates.changes.length; i++)
      {
        if (updates.changes[i] != null)
        {
	        var element = document.getElementById(updates.changes[i].name);
	        if (element != null)
	        {
	          if (element.tagName == "INPUT")
	          {
	            element.value = updates.changes[i].html;
	          }
	          else
	          {
		          if (updates.changes[i].html != null && updates.changes[i].html != "" )
		          {
		            element.innerHTML = updates.changes[i].html;
		          }
		          
		          if (typeof updates.changes[i].style != "undefined")
		          {
		            for (j = 0; j < updates.changes[i].style.length; j++)
		            {
		              //Konqueror does not process an eval properly (updates the element with focus)
		              if (updates.changes[i].style[j].name == "backgroundColor")
		              {
		                element.style.backgroundColor = updates.changes[i].style[j].value;
		              }
	                else if (updates.changes[i].style[j].name == "display")
	                {
	                  element.style.display = updates.changes[i].style[j].value;
	                }
	                else if (updates.changes[i].style[j].name == "visibility")
	                {
	                  element.style.visibility = updates.changes[i].style[j].value;
	                }
		              else
		              {
		                alert ("Unsupported style: " + updates.changes[i].style[j].name);
		              }
		            }
		          }
	          }
          }
        }
      }
    }
    
    if (typeof updates.images != "undefined")
    {
      for (i = 0; i < updates.images.length; i++)
      {
        var image = document.getElementById(updates.images[i].name);
        if (image != null)
        {
          image.src = updates.images[i].query;
        }
      }
    }
  }
}

function setRefreshEnabled()
{ if (!isThrottled || loadCompleteTime >= lastChangeTime)
  {
    doGetUpdates();
  }
}

function doGetUpdates()
{
	  frames['updateFrame'].document.open();
	  frames['updateFrame'].document.write(""
	         + "\<html\>\<head\>"
	         + "\<SCRIPT src=\"" + updates.jsp + "?time=" 
	         + new Date().getTime() + "&!{dataRefreshBean.refreshEnabled}=" + (updates.interval > 0)
	         + "\"\>\</SCRIPT\>"
	         + "\</head\>\</html\>");
	  frames['updateFrame'].document.close();
}

function tick(initial)
{
  if (allowRefresh && updates.interval > 0 && !pause)
  {
    if (!isThrottled || loadCompleteTime >= lastChangeTime)
    {
      doGetUpdates();
      setTimeout('tick()', minimumTime == -1 ? updates.interval : minimumTime);      
    }
    else
    {
      missed ++;
      if (missed > 15)
      {
        location = "?refresh";
      }
      else
      {
        setTimeout('tick()', 1000);
      }
    }
  }
}

function initialUpdate(throttle)
{
  if (typeof throttle != "undefined" && throttle)
  {
    isThrottled = true;
  }
  processUpdates();
  tick();
}

function adjustUpdateTime(loaded)
{
  if (lastChangeTime != -1)
  {
    loadCompleteTime = new Date().getTime();
    updateTime = loadCompleteTime - lastChangeTime;
    if (minimumTime == -1)
    {
      minimumTime = updates.interval;
    }
    if (updateTime > (minimumTime - (minimumTime / 5)))
    {
      minimumTime = 5 * updateTime / 4;
    }
    else if (updateTime < minimumTime / 2)
    {
      minimumTime /= 2;
      if (minimumTime < updates.interval)
      {
        minimumTime = updates.interval;
      }
    }
  }
}
